@@ -841,6 +841,8 @@ CAMLprim value core_unix_sysconf(value v_name)
 #endif
     case 14 : name = _SC_IOV_MAX; break;
     case 15 : name = _SC_CLK_TCK; break;
+    case 16 : name = _SC_NPROCESSORS_CONF; break;
+    case 17 : name = _SC_NPROCESSORS_ONLN; break;
     default :
       /* impossible */
       caml_failwith("unix_sysconf: unknown sum tag");
@@ -959,10 +961,13 @@ CAMLprim value core_unix_getgrouplist(value v_user, value v_group)
 {
   int n;
   int ngroups = NGROUPS_MAX;
-  gid_t groups[NGROUPS_MAX];
+  gid_t* groups;
   value ret;
   char *c_user;
 
+  groups = (gid_t*) malloc(NGROUPS_MAX * sizeof(gid_t));
+  if (groups == NULL) abort();
+
   assert(Is_block(v_user) && Tag_val(v_user) == String_tag);
   assert(!Is_block(v_group));
 
@@ -973,13 +978,19 @@ CAMLprim value core_unix_getgrouplist(value v_user, value v_group)
     free(c_user);
   caml_leave_blocking_section();
 
-  if (n == -1) uerror ("getgrouplist", Nothing);
+  if (n == -1) {
+    free(groups);
+    uerror ("getgrouplist", Nothing);
+  }
 
   ret = caml_alloc(n, 0);
   for (n = n - 1; n >= 0; n--) {
+    assert(n < NGROUPS_MAX);
     Store_field(ret, n, Val_long(groups[n]));
   }
 
+  free(groups);
+
   return ret;
 }
 
@@ -1605,6 +1616,33 @@ core_unix_getifaddrs(value v_unit)
   CAMLreturn(head);
 }
 
+CAMLprim value core_unix_all_ifnames(value v_unit) {
+  CAMLparam1(v_unit);
+  CAMLlocal3(head, next, name);
+
+  struct if_nameindex *intf_all;
+  struct if_nameindex *intf;
+
+  caml_release_runtime_system();
+  intf_all = if_nameindex();
+  caml_acquire_runtime_system();
+
+  head = Val_unit; /* [] */
+  for (intf = intf_all; intf != NULL && intf->if_index != 0; intf++) {
+    next = caml_alloc(2, 0 /* :: */);
+    name = caml_copy_string(intf->if_name);
+    Store_field(next, 0, name);
+    Store_field(next, 1, head);
+    head = next;
+  }
+
+  caml_release_runtime_system();
+  if_freenameindex(intf_all);
+  caml_acquire_runtime_system();
+
+  CAMLreturn(head);
+}
+
 #include <arpa/inet.h>
 
 CAMLprim value core_unix_inet4_addr_of_int32(value v) {
